Package: dann
Type: Package
Title: Discriminant Adaptive Nearest Neighbor Classification
Version: 1.3.0
Authors@R: 
    person("Greg", "McMahan", , "gmcmacran@gmail.com", role = c("aut", "cre"))
Description: Discriminant Adaptive Nearest Neighbor Classification is a 
    variation of k nearest neighbors where the shape of the neighborhood is 
    data driven. The neighborhood is elongated along class boundaries and 
    shrunk in the orthogonal direction. This package implements dann and 
    sub_dann from Hastie (1996) 
    <https://web.stanford.edu/~hastie/Papers/dann_IEEE.pdf>.
License: MIT + file LICENSE
Encoding: UTF-8
Imports: stats (>= 3.5.3), tibble (>= 2.1.1), ggplot2 (>= 3.1.1),
        stringr (>= 1.4.0), rlang (>= 1.0.0), fpc (>= 2.1-11.1), Rcpp
        (>= 1.0.1), hardhat
RoxygenNote: 7.3.3
Suggests: testthat (>= 3.0.0), rmarkdown (>= 1.18), mlbench (>= 2.1-1),
        dplyr (>= 0.8.0.1), magrittr (>= 1.5), recipes
LinkingTo: Rcpp, RcppArmadillo
Config/testthat/edition: 3
URL: https://github.com/gmcmacran/dann
BugReports: https://github.com/gmcmacran/dann/issues
NeedsCompilation: yes
Packaged: 2026-08-28 02:51:40 UTC; ixi_eulogy_ixi
Author: Greg McMahan [aut, cre]
Maintainer: Greg McMahan <gmcmacran@gmail.com>
Repository: CRAN
Date/Publication: 2026-08-28 13:50:02 UTC
Built: R 4.5.2; x86_64-apple-darwin20; 2026-08-28 14:51:25 UTC; unix
Archs: dann.so.dSYM
