# Generated by roxygen2: do not edit by hand

S3method(dann,data.frame)
S3method(dann,default)
S3method(dann,formula)
S3method(dann,matrix)
S3method(dann,recipe)
S3method(graph_eigenvalues,data.frame)
S3method(graph_eigenvalues,default)
S3method(graph_eigenvalues,formula)
S3method(graph_eigenvalues,matrix)
S3method(graph_eigenvalues,recipe)
S3method(predict,dann)
S3method(predict,sub_dann)
S3method(print,dann)
S3method(print,sub_dann)
S3method(sub_dann,data.frame)
S3method(sub_dann,default)
S3method(sub_dann,formula)
S3method(sub_dann,matrix)
S3method(sub_dann,recipe)
export(dann)
export(dann_get_threads)
export(dann_has_openmp)
export(dann_set_threads)
export(graph_eigenvalues)
export(sub_dann)
importFrom(Rcpp,sourceCpp)
importFrom(rlang,.data)
importFrom(stats,predict)
useDynLib(dann, .registration = TRUE)
