# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

#' @keywords internal
#' Full DANN prediction loop in C++ with OpenMP parallelization.
#' Replicates the behavior of the R dann_predict_base outer loop.
dann_predict_all_C <- function(xTrain, yTrain, xTest, k, neighborhood_size, epsilon, y_class_precedence, unique_classes, probability) {
    .Call(`_dann_dann_predict_all_C`, xTrain, yTrain, xTest, k, neighborhood_size, epsilon, y_class_precedence, unique_classes, probability)
}

